﻿using Kivii;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Test
{
    public partial class FrmMain : Form
    {
        ServerEventsClient _client = null;
        public FrmMain()
        {
            InitializeComponent();
        }

        private void btnConnect_Click(object sender, EventArgs e)
        {
            if (_client != null)//断开操作
            {
                _client.Stop();
                _client.Dispose();
                _client = null;
                btnConnect.Text = "连接";
                return;
            }
            btnConnect.Enabled=false;
            _client = new ServerEventsClient(tbxServiceUrl.Text,"home");
            var auth = new Authenticate { UserName =tbxUserName.Text, Password = tbxPassword.Text, provider = "kivii" };
            var task=_client.ServiceClient.PostAsync(auth);
            task.Success(()=>{
                _client.OnConnect = onConnect;
                _client.OnCommand = onCommand;
                _client.OnMessage = onMessage;
                _client.OnException = onException;
                _client.OnHeartbeat = onHeartbeat;
                _client.Connect();
            }, true);
            task.Error((ex)=> {
                btnConnect.Enabled = true;
                _client.Dispose();
                _client = null;
                rtbMessage.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}:Exception {ex.Message}{Environment.NewLine}{Environment.NewLine}");
            }, true);

        }
        private void onConnect(ServerEventConnect connection)
        {
            if (InvokeRequired)
            {
                Invoke(new Action(() => onConnect(connection)));
                return;
            }
            btnConnect.Enabled = true;
            btnConnect.Text = "断开";
            tbxFullName.Text = connection.DisplayName;
            tbxSubscriptionId.Text = connection.Id;
            tbxChannel.Text = connection.Channels?.Join(",");

            tbxConnectedTime.Text = DateTime.Now.ToString("HH:mm:ss");
            tbxConnectedTime.BackColor = tbxConnectedTime.BackColor != Color.MistyRose ? Color.MistyRose : Color.YellowGreen;

            rtbMessage.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}:onConnect {connection.Json}{Environment.NewLine}{Environment.NewLine}");
        }
        private void onHeartbeat()
        {
            if (InvokeRequired)
            {
                Invoke(new Action(onHeartbeat));
                return;
            }
            tbxHeartBeatTime.BackColor = tbxHeartBeatTime.BackColor != Color.MistyRose ? Color.MistyRose : Color.YellowGreen;
            tbxHeartBeatTime.Text = DateTime.Now.ToString("HH:mm:ss");
        }

        private void onException(Exception ex)
        {
            if (InvokeRequired)
            {
                Invoke(new Action(()=>onException(ex)));
                return;
            }
            rtbMessage.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}:onException {ex.Message}{Environment.NewLine}{Environment.NewLine}");
        }

        private void onMessage(ServerEventMessage message)
        {
            if (InvokeRequired)
            {
                Invoke(new Action(() => onMessage(message)));
                return;
            }
            rtbMessage.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}:onMessage {message.Json}{Environment.NewLine}{Environment.NewLine}");
        }

        private void onCommand(ServerEventMessage command)
        {
            if (InvokeRequired)
            {
                Invoke(new Action(() => onCommand(command)));
                return;
            }
            rtbMessage.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}:onCommand {command.Json}{Environment.NewLine}{Environment.NewLine}");
        }



        private void btnSendMessage_Click(object sender, EventArgs e)
        {
        }

        private void btnGet_Click(object sender, EventArgs e)
        {
            var users = _client.GetChannelSubscribers();
            var id = _client.SubscriptionId;
        }

        private void btnUpdateChannel_Click(object sender, EventArgs e)
        {
            btnUpdateChannel.Enabled = false;
            var task=_client.SubscribeToChannelsAsync(tbxChannel.Text.Split(new char[1] { ',' }, StringSplitOptions.RemoveEmptyEntries));
            task.Success(()=>{
                btnUpdateChannel.Enabled = true;
            });
            task.Error((ex)=> {
                btnUpdateChannel.Enabled = true;
                rtbMessage.AppendText($"{DateTime.Now.ToString("HH:mm:ss")}:onException {ex.Message}{Environment.NewLine}{Environment.NewLine}");
            });

        }
    }
}
